#include "GpBaseClass.h"

GpBaseClass::GpBaseClass()
{
  currentsurface=0;
  GpGraphicModeSet(8, NULL);
  GpLcdSurfaceGet(&gpDraw[0], 0);
  GpLcdSurfaceGet(&gpDraw[1], 1);
  GpSurfaceSet(&gpDraw[0]);
  GpLcdEnable();
  swapdelay=20;
  currentcolor=1;
  keystatus=GPC_VK_NONE;
  GpSrand(GpTickCountGet());
}


GpBaseClass::~GpBaseClass()
{
}


void GpBaseClass::Run()
{
  int cur_tick;

  while(1)
  {
    cur_tick=GpTickCountGet();
    this->Paint();
    HandleKeys();
    GpSurfaceFlip(&gpDraw[currentsurface]);
    while ((GpTickCountGet()-cur_tick)<swapdelay);
    currentsurface=1-currentsurface;
  }
}


void GpBaseClass::Paint()
{
  ClrScreen();
}


void GpBaseClass::GpSetPaletteEntry ( u8 i, u8 r, u8 g, u8 b )
{
  GP_PALETTEENTRY entry = GP_RGB24(r,g,b);
  GpPaletteEntryChange ( i, 1, &entry, 0 );
}


void GpBaseClass::Delay(int delay)
{
  int cur_tick = GpTickCountGet();
  while ((GpTickCountGet()-cur_tick)<delay);
}


void GpBaseClass::HandleKeys()
{
  int keydata=GpKeyGet() & 0x3FF;
  int mask = 0x01;
  int key,oldkey;

  while (mask!=0x400)
  {
    key=keydata&mask;
    oldkey=keystatus&mask;
    if (key!=oldkey)
    {
      if (key!=0)
        KeyDown(mask);
      else
        KeyUp(mask);
    }
    mask=mask<<1;
  }
  keystatus=keydata;
}


void GpBaseClass::KeyDown(int key)
{
//  char buf[16];

//  sprintf(buf,"KEYDOWN = %04d",key);
//  DrawString(buf,0,0,1);
}


void GpBaseClass::KeyUp(int key)
{
//  char buf[16];

//  sprintf(buf,"KEYUP = %04d",key);
//  DrawString(buf,0,10,1);
}


void GpBaseClass::DrawPolygon(int *x, int *y, int num, int color)
{
  int col = currentcolor;
  int i;
  int *p_x = x;
  int *p_y = y;
  int oldx,oldy;

  if (color>=0)
    col=color;

  oldx=*x;
  oldy=*y;

  for (i=1; i<num; i++)
  {
    p_x++;
    p_y++;
    DrawLine(oldx,oldy,*p_x,*p_y,col);
    oldx=*p_x;
    oldy=*p_y;
  }
  DrawLine(*x,*y,*p_x,*p_y);
}


void GpBaseClass::DrawPolyline(int *x, int *y, int num, int color)
{
  int col = currentcolor;
  int i;
  int *p_x = x;
  int *p_y = y;
  int oldx,oldy;

  if (color>=0)
    col=color;

  oldx=*x;
  oldy=*y;

  for (i=1; i<num; i++)
  {
    p_x++;
    p_y++;
    DrawLine(oldx,oldy,*p_x,*p_y,col);
    oldx=*p_x;
    oldy=*p_y;
  }
}
