//-------------------------------------------------------------------
//
// A Baseclass for GP32 graphics handling (for demos and games)
// (c)2003 by Brian Postma - B.Postma@HetNet.nl - www.brianpostma.com
//
//-------------------------------------------------------------------
//
// Usage: Inherit from this class. Start the class by calling the
//        Run() method. This method will perform surface swapping and
//        call the Paint() method in an endless loop. The user should
//        place the game/demo handling in the Paint() method which
//        should be overloaded in the derived class.
//
//-------------------------------------------------------------------



#ifndef __GPBASECLASS_H_INCLUDED__
#define __GPBASECLASS_H_INCLUDED__

#include "defines.h"
#include "gpgraphic.h"
#include "gpfont.h"
#include "gpstdlib.h"
#include "stdio.h"
#include "string.h"

/* Converts 8bit rgb values to a GP32 palette value */
#define GP_RGB24(r,g,b) (((((r>>3))&0x1f)<<11)|((((g>>3))&0x1f)<<6)|((((b>>3))&0x1f)<<1))

class GpBaseClass
{ 
public:
  GpBaseClass();
  ~GpBaseClass();
  void GpSetPaletteEntry ( u8 i, u8 r, u8 g, u8 b );
  void Delay(int delay);
  void Run();
  inline float Random()
  {
    return ((float)GpRand())/65536.0;
  }
  inline void SetDelay(int delay)
  {
    swapdelay=delay;
  }
 
protected:
  virtual void Paint();
  virtual void KeyDown(int key);
  virtual void KeyUp(int key);

  void DrawPolygon(int *x, int *y, int num, int color=-1);
  void DrawPolyline(int *x, int *y, int num, int color=-1);

  inline GPDRAWSURFACE *GetDrawSurface()
  {
    return &gpDraw[currentsurface];
  }
  inline GPDRAWSURFACE *GetVisibleSurface()
  {
    return &gpDraw[1-currentsurface];
  }
  inline void DrawString(char *str, int x, int y)
  {
    GpTextOut(NULL, &gpDraw[currentsurface], x, y, str, currentcolor);
  }
  inline void DrawString(char *str, int x, int y, int color)
  {
    GpTextOut(NULL, &gpDraw[currentsurface], x, y, str, color);
  }
  inline void SetColor(int color)
  {
    currentcolor=color;
  }
  inline void ClrScreen()
  {
    GpRectFill(NULL, &gpDraw[currentsurface], 0, 0, gpDraw[currentsurface].buf_w, gpDraw[currentsurface].buf_h,0);
  }
  inline void DrawLine(int x1, int y1, int x2, int y2)
  {
    GpLineDraw(&gpDraw[currentsurface],x1,y1,x2,y2,currentcolor);
  }    
  inline void DrawLine(int x1, int y1, int x2, int y2, int color)
  {
    GpLineDraw(&gpDraw[currentsurface],x1,y1,x2,y2,color);
  }    
  inline void FillRect(int x, int y, int w, int h)
  {
    GpRectFill(NULL, &gpDraw[currentsurface], x, y, w, h,currentcolor);
  }
  inline void FillRect(int x, int y, int w, int h, int color)
  {
    GpRectFill(NULL, &gpDraw[currentsurface], x, y, w, h, color);
  }
  inline void DrawRect(int x, int y, int w, int h)
  {
    GpRectDraw( &gpDraw[currentsurface], x, y, x+w, y+h,currentcolor);
  }
  inline void DrawRect(int x, int y, int w, int h, int color)
  {
    GpRectDraw(&gpDraw[currentsurface], x, y, x+w, y+h,color);
  }
  inline void FillRectAbs(int x1, int y1, int x2, int y2)
  {
    GpRectFill(NULL, &gpDraw[currentsurface], x1, y1, x2-x1, y2-y1,currentcolor);
  }
  inline void FillRectAbs(int x1, int y1, int x2, int y2, int color)
  {
    GpRectFill(NULL, &gpDraw[currentsurface], x1, y1, x2-x1, y2-y1, color);
  }
  inline void DrawRectAbs(int x1, int y1, int x2, int y2)
  {
    GpRectDraw(&gpDraw[currentsurface], x1, y1, x2, y2,currentcolor);
  }
  inline void DrawRectAbs(int x1, int y1, int x2, int y2, int color)
  {
    GpRectDraw(&gpDraw[currentsurface], x1, y1, x2, y2,color);
  }
  inline int GetScreenWidth()
  {
    return gpDraw[0].buf_w;
  }
  inline int GetScreenHeight()
  {
    return gpDraw[0].buf_h;
  }
  inline int GetKeys()
  {
    return keystatus;
  }
  
private:
  void HandleKeys();
  GPDRAWSURFACE gpDraw[2];
  int currentsurface;
  int swapdelay;
  int currentcolor;
  int keystatus;
};

#endif 
