// Java Tetris
// Created on a boring sunday.....
//
// (c)2001 by Brian Postma
// e-mail: B.Postma@HetNet.nl
// homepage: http://www.homepages.hetnet.nl/~brianpostma
//

#include "Jatris.h"

Jatris::Jatris()
{
    GpSetPaletteEntry(0,  0, 0, 0);
    GpSetPaletteEntry(1,  255,255,255);
    GpSetPaletteEntry(barcolor, 128,255,64);
    GpSetPaletteEntry(textcolor1, 96,128,255);
    GpSetPaletteEntry(textcolor2, 255,160,64);
    GpSetPaletteEntry(16, 255,0,0);
    GpSetPaletteEntry(17, 0,255,0);
    GpSetPaletteEntry(18, 0,0,255);
    GpSetPaletteEntry(19, 255,255,0);
    GpSetPaletteEntry(20, 255,0,255);
    GpSetPaletteEntry(21, 255,0,255);

    blocks[0]=0;
    blocks[1]=16;
    blocks[2]=17;
    blocks[3]=18;
    blocks[4]=19;
    blocks[5]=20;
    blocks[6]=21;
    ingame=false;
    fast=false;
    objectrotationd=0;

    gameInit();
}


Jatris::~Jatris()
{
}


void Jatris::gameInit()
{
    short i,j;

    for (i=0; i<xblocks; i++)
    {
      for (j=0; j<yblocks; j++)
      {
        screendata[i][j]=0;
      }
    }

    score=0;
    emptyline=-1;

    newObject();
    fast=false;
    curcount=maxcount;
}


void Jatris::newObject()
{
    short i;
    int   y;

    objectx=xblocks/2-1;
    objectdx=0;
    objecty=0;
    objecttype=(short)(Random()*itemcount);
    if (objecttype>=itemcount)
      objecttype=itemcount-1;

    objectptr=(short)(objecttype*itemlen);
    checkptr=(short)(objecttype*itemlen);

    objectcolor=(short)(Random()*maxcolors+1);
    if (objectcolor>maxcolors)
      objectcolor=maxcolors;
    objectrotation=0;
    count=maxcount;

    // check if game has ended
    for (i=0; i<4; i++)
    {
      y=items[objectptr+i*2+1];
      if (y>=0 && screendata[objectx+items[objectptr+i*2]][y]!=0)
      {
        ingame=false;
      }
    }
}


void Jatris::KeyDown(int key)
{
    if (ingame)
    {
      if (key == GPC_VK_LEFT)
      {
        objectdx=-1;
      }
      else if (key == GPC_VK_RIGHT)
      {
        objectdx=1;
      }
      else if (key == GPC_VK_FA)
      {
        objectrotationd=1;
      }
      else if (key == GPC_VK_FB)
      {
        fast=true;
      }
      else if (key == GPC_VK_SELECT)
      {
        ingame=false;
      }
    }
    else
    {
      if (key == GPC_VK_START)
      {
        ingame=true;
        gameInit();
      }
    }
}


void Jatris::KeyUp(int key)
{
    if (key == GPC_VK_FB)
    {
      fast=false;
    }
}


void Jatris::Paint()
{
    ClrScreen();

    if (ingame)
    {
      playGame();
    }
    else
    {
      drawBars();
      drawBlocks();
    }
    showIntro();
    showScore();
}


void Jatris::playGame()
{
    bool bottomreached=false;
    bool stillscrolling=false;

    if (emptyline<0)
    {
      bottomreached=drawObject();
    }
    else
    {
      scrollDown();
      stillscrolling=true;
    }
    drawBars();
    drawBlocks();
    if (stillscrolling || bottomreached)
    {
      checkFull();
    }
}


void Jatris::showIntro()
{
    char s1[]="JATRIS";
    char s2[]="(c) by Brian Postma";
    char s3[]="B.Postma@HetNet.nl";
    char s4[]="www.brianpostma.com";
    char s5[]="Use Left,Right";
    char s6[]="[A] to rotate";
    char s7[]="[B] for quick drop";
 
    DrawString(s1,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s1))/2+xblocks*blocksize,50,textcolor1);
    DrawString(s2,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s2))/2+xblocks*blocksize,70,1);
    DrawString(s3,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s3))/2+xblocks*blocksize,90,textcolor2);
    DrawString(s4,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s4))/2+xblocks*blocksize,105,textcolor2);
    DrawString(s5,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s5))/2+xblocks*blocksize,130,textcolor1);
    DrawString(s6,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s6))/2+xblocks*blocksize,145,textcolor1);
    DrawString(s7,((GetScreenWidth()-xblocks*blocksize)-8*strlen(s7))/2+xblocks*blocksize,160,textcolor1);
}


void Jatris::drawBars()
{
    FillRect(0,0,barwidth,blocksize*yblocks,barcolor);
    FillRect(barwidth+blocksize*xblocks,0,barwidth,blocksize*yblocks,barcolor);
    FillRect(0,blocksize*yblocks,xblocks*blocksize+2*barwidth,barwidth,barcolor);
}


bool Jatris::drawObject()
{
    short	i;
    bool	bottomreached=false;
    int		x,y,checkx,checky;

    // clear old
    for (i=0; i<4; i++)
    {
      x=objectx+items[objectptr+i*2+objectrotation*itemrotlen];
      y=objecty+items[objectptr+i*2+objectrotation*itemrotlen+1];
      checkx=objectx+checks[objectptr+i*2+objectrotation*itemrotlen];
      checky=objecty+checks[objectptr+i*2+objectrotation*itemrotlen+1];
      if (y>=0)
        screendata[x][y]=0;
      if (screendata[checkx][checky]!=0)
        bottomreached=true;

    }

    if (!bottomreached)
    {
      count--;
      if (count<=0 || fast)
      {
        objecty++;
        count=curcount;
      }
      checkRotation();
      objectdx=0;
      objectrotationd=0;
    }

    // draw new
    for (i=0; i<4; i++)
    {
      x=objectx+items[objectptr+i*2+objectrotation*itemrotlen];
      y=objecty+items[objectptr+i*2+objectrotation*itemrotlen+1];

      if (y>=0)
        screendata[x][y]=objectcolor;
      if (y>=(yblocks-1))
        bottomreached=true;
    }
    if (bottomreached)
    {
      score++;
      newObject();
    }
    return bottomreached;
}


void Jatris::checkRotation()
{
    int		dummyx;
    int		dummyrot;
    int		x,y;
    short	i;
    bool	cando=true;

    dummyrot=(objectrotation+objectrotationd)%4;
    dummyx=objectx+objectdx;

    // make sure the part doesn't rotate of the playscreen
    for (i=0; i<4; i++)
    {
      x=dummyx+items[objectptr+i*2+dummyrot*itemrotlen];
      if (x>=xblocks)
        dummyx-=(x-xblocks+1);
      else if (x<0)
        dummyx-=x;
    }

    for (i=0; (i<4 && cando); i++)
    {
      x=dummyx+items[objectptr+i*2+dummyrot*itemrotlen];
      y=objecty+items[objectptr+i*2+dummyrot*itemrotlen+1];
      if (y>=0)
        cando=cando&&(screendata[x][y]==0);
      if (y>=yblocks || x<0 || x>=xblocks)
        cando=false;
    }
    if (cando)
    {
      objectrotation=dummyrot;
      objectx=dummyx;
    }
}


void Jatris::drawBlocks()
{
    short x,y;

    for (x=0; x<xblocks; x++)
    {
      for (y=0; y<yblocks; y++)
      {
        DrawRect(x*blocksize+barwidth,y*blocksize,blocksize-1,blocksize-1,blocks[screendata[x][y]]);
        FillRect(x*blocksize+barwidth+3,y*blocksize+3,blocksize-6,blocksize-6,blocks[screendata[x][y]]);
      }
    }
}


void Jatris::checkFull()
{
    short	x,y;
    bool	found=false;

    for (y=yblocks-1; ( y>=0 && !found); y--)
    {
      found=true;
      for (x=0; x<xblocks; x++)
      {
        if (screendata[x][y]==0)
          found=false;
      }
      if (found)
      {
        score+=10;

	// increase speed when you've got a lot of points
        if (score>800)
          curcount=1;
        else if (score>600)
          curcount=2;
        else if (score>400)
          curcount=3;
        else if (score>200)
          curcount=4;

        for (x=0; x<xblocks; x++)
        {
          screendata[x][y]=0;
        }
        emptyline=y;
      }
    }
}


void Jatris::scrollDown()
{
    short x,y;

    for (y=emptyline; y>0; y--)
    {
      for (x=0; x<xblocks; x++)
      {
        screendata[x][y]=screendata[x][y-1];
      }
    }
    for (x=0; x<xblocks; x++)
    {
      screendata[x][0]=0;
    }
    emptyline=-1;
}


void Jatris::showScore()
{
    char buf[16];

    sprintf(buf,"Score: %05d",score);
    DrawString(buf,width/2-40,(yblocks+1)*blocksize+2);
}
